--[[ PotionMaker by MrInfernal  v0.3

    Can search by potion name or effect, then select a number to make
    Will purchase the required number of ingredients and then craft

    ToDo:
    Create Potion table data
    Add TLP recipes
    Check Tradeskill Depot
    Purchase 25% extra to handle failures
    Create events to: track success and failures, if number of potions made, if out of money, if out of ingredients
    Get more money if needed from bank
    Get the ingredients to make more potions until complete
    Sell back the unused ingredients
    Check for and equip trophy. Re-equip weapon afterwards
    Check if recipe known
    If not known, add ingredients and craft manually until recipe learned
    Add suspensions and dyes to potions.lua
    Check for Throne of Heroes if needed
    Add support for extra perk inventory slots
]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- set variables
Potions = require('potions')
local numPot = '1' -- How many potions to make
local typeAndEffects = {{type='Buff', effects={'Damage Shield', 'Everlasting Breath', 'Group Everlasting Breath', 'Group Shrink', 
'Group Spirit of the Wolf', 'Grow', 'Group Levitation', 'Haste', 'Levitation', 'Shrink', 'Spirit of the Wolf'}},
{type='Buffs: Divination', effects={'Faerune', 'Improve Archery', 'Improved Invisibility', 'Infravision', 'Invisibility', 'Invis vs Undead', 'See Invisible', 
'Ultravision'}},
{type='Heal/Regen', effects={'Cure Disease', 'Cure Poison', 'Heal', 'Heal over Time', 'HP and Mana Regen', 'HpToMana', 'ManaToHP', 'Mana Regen', 
'Regeneration'}},
{type='Increase', effects={'Beneficial Duration', 'Detrimental Duration', 'Spell Distance'}},
{type='Illusion: B-G', effects={'Barbarian', 'Dark Elf', 'Drakkin', 'Dwarf', 'Erudite', 'Gnome', 'Guktan'}},
{type='Illusion: H-V', effects={'Halfling', 'Half Elf', 'High Elf', 'Human', 'Iksar', 'Ogre', 'Troll', 'Wood Elf', 'Vah Shir'}},
{type='Illusion: Enemy', effects={'Basilisk', 'Bear', 'Earth Elemental', 'Fairy', 'Goblin', 'Imp', 'Kobold', 'Skeleton'}},
{type='Reduce Mana', effects={'Beneficial Spells', 'Chromatic Spells', 'Cold Spells', 'Disease Spells', 'Fire Spells', 'Magic Spells', 'Poison Spells'}},
{type='Resist', effects={'Resist Cold', 'Resist Disease', 'Resist Fire', 'Resist Magic', 'Resist Poison'}},
{type='Spell', effects={'Cancel Magic', 'Gate', 'Gate: Everfrost', 'Gate: Field of Bone', 'Gate: Innothule', "Gate: Shadeweaver's Thicket", 
'Gate: Twilight Sea', 'Nullify Magic', 'Shadow Step'}},
{type='Stat', effects={'Increase Agility', 'Increase Charisma', 'Increase Dexterity', 'Increase HP and AC', 'Increase Intelligence', 
'Increase Mana', 'Increase Stamina', 'Increase Strength', 'Increase Wisdom'}}
    }
local levelSelect = 1
local potLevel = {'Any', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X', 'XI', 'XII', 'XIII',
                'XIV', 'XV', 'XVI', 'XVII', 'XVIII', 'XIX', 'XX', 'XI', 'XII'}
local selectedTypeIndex = 1
local selectedEffectIndex = 1
local selectedPotionIndex = 1
local typeSelect = ''
local effectSelect = ''
local lookupResults = {}
local selectedPotion = {}
local alcSkill = mq.TLO.Me.Skill("Alchemy")()
local myCash = mq.TLO.Me.Cash()/1000
local useCWTN = false
local gatedToPOK = false
local onlyVendor = false

-- Functions
local function goToVendor(zone)
    -- Check if bound in PoK and then gates. Otherwise, runs to PoK
    if zone ~= 'poknowledge' then
        if mq.TLO.Me.ZoneBound() == 'The Plane of Knowledge' then
            if mq.TLO.Cast.Ready('Gate|alt') then
                mq.cmd('/squelch /casting "gate" alt')
                print('Gating to Plane of Knowledge')
                gatedToPOK = true
            else
                if useCWTN then
                    mq.cmd('/squelch /shm pause on')
                end
                mq.cmd('/squelch /casting "gate"')
                print('Gating to Plane of Knowledge')
                gatedToPOK = true
            end
        else
        mq.cmd('/travelto poknowledge @ Alchemist Redsa')
        print('Traveling to Alchemist Redsa in the Plane of Knowledge!')
        end
    else
        mq.cmd('/easyfind Alchemist Redsa')
    end
end

local function checkCWTN()
    if mq.TLO.Plugin('shaman').IsLoaded() and mq.TLO.Me.Class.Name() == 'Shaman' then
        useCWTN = true
    else
        useCWTN = false
    end
end

local function gatedToPOKEvent()
    -- After gating, this turns the MQ2Shaman back on if it was paused
    -- Also continues travel to the alchemy merchant
    if gatedToPOK then
        gatedToPOK = false
    end
    if useCWTN and mq.TLO.CWTN.Paused() then
        mq.cmd('/squelch /shm pause off')
    end
    mq.cmd('/easyfind Alchemist Redsa')
end

local function potLookup(type, effect, level, onlyVendor)
    local results = {}

    for _, pot in ipairs(Potions) do
        if level > 1 then
            if onlyVendor then
                if pot.type == type and pot.effect == effect and pot.level == tonumber(level)-1 and pot.vendor then
                    table.insert(results, pot)
                end
            else
                if pot.type == type and pot.effect == effect and pot.level == tonumber(level)-1 then
                table.insert(results, pot)
                end
            end
        else
            if onlyVendor then
                if pot.type == type and pot.effect == effect and pot.vendor then
                    table.insert(results, pot)
                end
            else
                if pot.type == type and pot.effect == effect then
                table.insert(results, pot)
                end
            end  
        end
    end

    return results
end

local function matList(potion)
    local matsCount = {}

    -- Check for potion ingredients as well as quantity
    for _, mat in ipairs(potion.mats) do
        if matsCount[mat] then
            matsCount[mat] = matsCount[mat] + 1
        else
            matsCount[mat] = 1
        end
    end

    return matsCount
end

local function makePotion(potion, numPot)
    ------ Buy the materials
    -- ToDo: Check for mats in inventory, bank, and tradeskill depot

    -- How many potions to make
    local multiplier = tonumber(numPot)
    local matsCount = matList(potion)
    local delay = 80

    -- Multiply counts if needed
    for mat, count in pairs(matsCount) do
        matsCount[mat] = count * multiplier
    end
    
    -- Open the merchant window
    mq.cmd('/tar Alchemist Redsa')
    if not mq.TLO.Merchant.Open() then
        mq.cmd('/timed 5 /invoke ${Merchant.OpenWindow}')
    end

    -- Buy the required ingredients
    if tostring(mq.TLO.Merchant.ItemsReceived) == 'TRUE' then
        delay = 10
    end

    -- Check that the item doesn't begin with an *
    -- Those items are not sold
    for mat, count in pairs(matsCount) do
        if not mat:match("^%*") then
            mq.cmdf('/timed %d /invoke ${Merchant.SelectItem[=%s]}', delay, mat)
            delay = delay + 5
            mq.cmdf('/timed %d /invoke ${Merchant.Buy[%d]}', delay, count)
            delay = delay + 5
        end
    end

    --Check for and open medicine bag in inventory
    local bagsToFind = {'Foldable Reinforced Medicine Bag', 'Foldable Medicine Bag', 'Reinforced Medicine Bag', 'Medicine Bag'}
    local medBag = ''
        
    for packIndex = 1, 10 do
        local packName = "pack" .. packIndex
        local packItem = mq.TLO.Me.Inventory(packName).Name()

        -- Check if packItem's name is in the list of bags to find
        if table.concat(bagsToFind, ","):find(packItem) then
            medBag = packItem
        end
    end

    if medBag ~= '' and not mq.TLO.Window('TradeskillWnd').Open() then
        mq.cmdf('/timed %d /itemnotify "%s" rightmouseup', delay, medBag)
        delay = delay + 5
    end

    -- Split the potion name into words for search
    local words = {}
    for word in potion.name:gmatch("%S+") do
        table.insert(words, word)
    end

    -- Get the next-to-last word and enter in Search bar
    local result = #words >= 2 and words[#words - 1] or potion.name
    mq.cmdf('/timed %d /invoke ${Window[TradeskillWnd/COMBW_SearchTextEdit].SetText["%s"]}', delay, result)
    delay = delay + 5

    -- Check the Make All button if it isn't
    if not mq.TLO.Window('TradeskillWnd/COMBW_MakeAllCheckbox').Checked() then
        mq.cmdf('/timed %d /notify TradeskillWnd COMBW_MakeAllCheckbox leftmouseup', delay)
        delay = delay + 5
    end

    delay = delay + 65 -- To allow the Search Recipe button to become selectable
    --Search the recipe window
    mq.cmdf('/multiline ; /timed %d /notify TradeskillWnd COMBW_SearchButton leftmouseup; /timed %d /echo Searching', delay, delay)
    delay = delay + 5

    -- Select the matching recipe
    local listNumber = string.format('${Window[TradeskillWnd/RecipeList].List[=%s]}', potion.name)
    mq.cmdf('/timed %d /notify TradeskillWnd RecipeList listselect %s', delay, listNumber)
    delay = delay + 10

    -- Combine the potions
    mq.cmdf('/timed %d /notify TradeskillWnd COMBW_CombineButton leftmouseup', delay)
end

-- function to run the window
local function potionMaker(open)
    -- Specify a default position/size
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 600, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)

    -- change the window size
    imgui.SetNextWindowSize(600, 300, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("Potion Maker", open)

    if not show then
        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Main window element area --
    imgui.Text("Alchemy Skill:")
    imgui.SameLine()
    imgui.Text(alcSkill)

    imgui.SameLine(175)
    imgui.Text('Plat: ')
    imgui.SameLine()
    imgui.Text(tostring(myCash))

    imgui.SameLine(550)
    if ImGui.Button("Go to vendor") then
        goToVendor(mq.TLO.Zone.ShortName())
    end
    imgui.SameLine()
    if ImGui.Button('Stop') then
        mq.cmd('/squelch /nav stop')
    end

    ImGui.Separator()
    -- Potion selection
    imgui.Columns(4)  -- Set up columns

    -- Display types in the first column
    imgui.Text("          Type")
    imgui.Text('\n')
    for i, pot in ipairs(typeAndEffects) do
        local isSelectedType = (i == selectedTypeIndex)
    
        -- Display the type as a selectable label
        if imgui.Selectable(pot.type, isSelectedType) then
            -- Handle selection logic for types here
            selectedTypeIndex = i
            typeSelect = pot.type
        end
    end
    
    imgui.NextColumn()
    
    -- Display effects in the second column for the selected type
    imgui.Text("         Effects")
    imgui.Text('\n')
    if selectedTypeIndex <= #typeAndEffects then
        local selectedEffects = typeAndEffects[selectedTypeIndex].effects
        for i, effect in ipairs(selectedEffects) do
            local isSelectedEffect = (i == selectedEffectIndex)
    
            -- Display the effect as a selectable label
            if imgui.Selectable(effect, isSelectedEffect) then
                -- Handle selection logic for effects here
                selectedEffectIndex = i
                effectSelect = effect
            end
        end
    end

    imgui.NextColumn()

    imgui.Text('            Level')
    imgui.Text('\n')

    levelSelect = imgui.Combo(" ", levelSelect, potLevel, #potLevel)

    imgui.NextColumn()

    imgui.Text('\n\n')

    if imgui.Button('Lookup') then
        lookupResults = potLookup(typeSelect, effectSelect, levelSelect, onlyVendor)
    end
    imgui.SameLine()
    if imgui.Button('Reset') then
        selectedTypeIndex = 1
        selectedEffectIndex = 1
        levelSelect = 1
        lookupResults = {}
    end

    onlyVendor = imgui.Checkbox("Only Store bought", onlyVendor)

    imgui.Text('\n')
    imgui.Text('Quantity:    ')
    imgui.SameLine()
    numPot = imgui.InputText('', tostring(numPot))

    if imgui.Button('Make Potion') then
        if selectedPotion then
            makePotion(selectedPotion, numPot)
        else
            print('No potion selected!')
        end
    end

    imgui.Text('\n')

    imgui.Columns(1)  -- Reset columns

    ImGui.Separator()
    -- Only display results if any are available
    if lookupResults and #lookupResults > 0 then
        -- Potions found
        imgui.Text('Potions')
        imgui.Text('\n')
        imgui.Text('Name')
        imgui.SameLine(225)
        imgui.Text('Trivial')
        imgui.SameLine(300)
        imgui.Text('Yields')
        imgui.SameLine(350)
        imgui.Text('Materials')
        imgui.SameLine(425)
        imgui.Text('Storebought Materials')

        -- Display information about selected potions
        for i, potion in ipairs(lookupResults) do
            local label = potion.name
            local isSelected2 = selectedPotionIndex == i
            if imgui.Selectable(label, isSelected2) then
                selectedPotion = potion
                selectedPotionIndex = i
            end

            local materials = matList(potion)
            local materialInfo = ""
            for mat, count in pairs(materials) do
                materialInfo = materialInfo .. mat .. ": " .. count .. "\n"
            end

            imgui.SameLine(225)
            imgui.Text('   ' .. potion.trivial)

            imgui.SameLine(300)
            imgui.Text('     ' .. potion.yield)

            imgui.SameLine(370)
            imgui.HelpMarker(materialInfo)

            imgui.SameLine(425)
            local vendor = potion.vendor and 'Yes' or 'No'
            imgui.Text('               ' .. vendor)
        end
    end
    -- End of main window element area --

    -- Required for window elements
    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

checkCWTN()

mq.event('gated', 'You have entered The Plane of Knowledge#*#', gatedToPOKEvent)

local openGUI = true

ImGui.Register('potionMaker', function()
    openGUI = potionMaker(openGUI)
end)

while openGUI do
    mq.delay(1000) -- equivalent to '1s'
end